@@ -27,6 +27,7 @@
 #include "bug-buddy.h"
 #include "distribution.h"
 #include "proccess.h"
+#include "forbidden-words.h"
 
 #include <stdio.h>
 #include <unistd.h>
@@ -73,6 +74,7 @@
 
 static GOptionData gopt_data;
 static int bug_count = 0;
+static GHashTable *apps = NULL;
 
 static const GOptionEntry options[] = {
 	{ "name",       '\0', 0, G_OPTION_ARG_STRING,   &gopt_data.name,        N_("Name of contact"),                    N_("NAME") },
@@ -94,6 +96,9 @@ enum {
 };
 
 static void fill_stderr_info (GladeXML *xml);
+static void fill_custom_info (BugzillaApplication *app, GladeXML *xml);
+static void close_callback   (GtkWidget *widget, gpointer user_data);
+static void bug_buddy_quit   (GladeXML *xml);
 
 
 static void
@@ -122,6 +127,78 @@ buddy_error (GtkWidget *parent, const char *msg, ...)
 	g_free (s);
 }
 
+static void
+lock_text (GladeXML *xml)
+{
+	GtkTextView *text_view;
+	GtkTextBuffer *buffer;
+	GtkTextIter start;
+	GtkTextIter end;
+	static GtkTextTag *tag = NULL;
+	char *text;
+
+	text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
+	buffer = gtk_text_view_get_buffer (text_view);
+	gtk_text_buffer_get_start_iter (buffer, &start);
+	gtk_text_buffer_get_end_iter (buffer, &end);
+	text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+
+	if (!tag) {
+		GtkStyle *style = gtk_widget_get_style (GTK_WIDGET (text_view));
+		tag = gtk_text_buffer_create_tag (buffer, "lock_tag", 
+						  "editable", FALSE,
+						  /* I don't like how it looks like dimming also fg
+						  "foreground-gdk", &style->fg[GTK_STATE_INSENSITIVE], */
+						  "background-gdk", &style->bg[GTK_STATE_INSENSITIVE],
+						  NULL);
+	}
+
+	if (gtk_text_iter_forward_search (&start, "Backtrace was generated from",
+					  GTK_TEXT_SEARCH_TEXT_ONLY,
+					  NULL, &end, NULL)) {
+		gtk_text_iter_forward_line (&end);
+		gtk_text_buffer_apply_tag_by_name (buffer, "lock_tag", &start, &end);
+	}
+}
+
+
+static gboolean
+search_forbidden_words (GladeXML *xml)
+{
+	GtkTextView *text_view;
+	GtkTextBuffer *buffer;
+	int i;
+	gboolean found = FALSE;
+	static GtkTextTag *tag = NULL;
+
+	text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
+	buffer = gtk_text_view_get_buffer (text_view);
+	if (!tag) {
+		tag = gtk_text_buffer_create_tag (buffer, "forbbiden_tag",
+						  "foreground", "white",
+						  "background", "blue",
+						  NULL);  
+	}
+
+	for (i = 0; forbidden_words[i]; i++) {
+		GtkTextIter start;
+		GtkTextIter end;
+
+		gtk_text_buffer_get_start_iter (buffer, &start);
+		while (gtk_text_iter_forward_search (&start, forbidden_words[i],
+                                                     GTK_TEXT_SEARCH_TEXT_ONLY,
+                                                     &start, &end, NULL)) {
+			gtk_text_buffer_apply_tag_by_name (buffer, "forbbiden_tag", &start, &end);
+			start = end;
+			found = TRUE;
+		}
+	}
+
+	return found;
+}
+
+
+
 static void
 copy_review (GtkWidget *button, gpointer data)
 {
@@ -145,11 +222,13 @@ edit_review (GtkWidget *button, gpointer data)
 {
 	GtkTextView *text_view;
 	GladeXML *xml = (GladeXML*) data;
+	gboolean editable = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
 
 	text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
-	gtk_text_view_set_editable (text_view,
-				    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
-
+	gtk_text_view_set_editable (text_view, editable);
+	if (editable) {
+		lock_text (xml);
+	}
 
 }
 
@@ -161,6 +240,12 @@ close_review (GtkWidget *button, gpointer data)
 	gtk_widget_hide (review_dialog);
 }
 
+static gboolean
+delete_review (GtkWidget *widget, GdkEvent  *event, gpointer user_data)  
+{
+	gtk_widget_hide (widget);
+}
+
 static void
 show_review (GtkWidget *button, gpointer data)
 {
@@ -181,12 +266,14 @@ show_review (GtkWidget *button, gpointer data)
 		g_signal_connect (G_OBJECT (copy), "clicked", G_CALLBACK (copy_review), xml);
 		g_signal_connect (G_OBJECT (edit), "toggled", G_CALLBACK (edit_review), xml);
 		g_signal_connect (G_OBJECT (close), "clicked", G_CALLBACK (close_review), review_dialog);
+		g_signal_connect (G_OBJECT (review_dialog), "delete-event", G_CALLBACK (delete_review), NULL);
 		
 		initialized = TRUE;
 	}
 
-	gtk_widget_show (glade_xml_get_widget (xml, "review-dialog"));
+	lock_text (xml);
 
+	gtk_widget_show (glade_xml_get_widget (xml, "review-dialog"));
 }
 
 
@@ -308,10 +395,6 @@ network_error (SoupMessage *msg, GladeXML *xml)
 	int res;
 		
 	gtk_widget_hide (glade_xml_get_widget (xml, "progressbar"));
-	gtk_statusbar_push (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")),
-		    	    DONE, _("Done"));
-
-
 		
 	dialog = gtk_message_dialog_new_with_markup (NULL,
 						      GTK_DIALOG_MODAL,
@@ -345,7 +428,7 @@ network_error (SoupMessage *msg, GladeXML *xml)
 		g_free (filename);
 	}
 		
-	gtk_main_quit();
+	bug_buddy_quit (xml);
 	return;
 }
 void
@@ -382,10 +465,8 @@ all_sent (GladeXML *xml)
 
 	/* connect the closed button to gtk_main_quit() */
 	close_button = glade_xml_get_widget (xml, "close-button");
-	g_signal_connect (close_button, "clicked", gtk_main_quit, NULL);
+	g_signal_connect (close_button, "clicked", G_CALLBACK (close_callback), xml);
 	gtk_widget_show (close_button);
-	gtk_statusbar_push (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")),
-			    DONE, _("Done"));
 }
 
 void
@@ -401,6 +482,8 @@ previous_sent (SoupMessage *msg, GladeXML *xml)
 void
 bug_sent (SoupMessage *msg, GladeXML *xml)
 {
+	GtkWidget *button;
+	GtkWidget *image;
 	char *text = NULL;
 	char *errmsg = NULL;
 	char *str = NULL;
@@ -410,6 +493,12 @@ bug_sent (SoupMessage *msg, GladeXML *xml)
 	GtkRequisition requisition;
 	GError *err = NULL;
 
+	button = glade_xml_get_widget (xml, "close-button");
+	gtk_button_set_label (GTK_BUTTON (button), _("_Close"));
+	gtk_button_set_use_underline (GTK_BUTTON (button), TRUE);
+		
+	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON), 
+	gtk_button_set_image (GTK_BUTTON (button), image);
 
 	if (SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code)) {
 		network_error (msg, xml);
@@ -644,15 +733,16 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GladeXML
 	gtk_text_buffer_get_end_iter (buffer, &end);
 	details_text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
 
-	final_text = g_strdup_printf ("What were you doing when the application crashed?\n%s\n\n\n%s", 
+	type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(xml), "type"));
+	final_text = g_strdup_printf ("%s%s\n\n\n%s", 
+				      type == BUG_TYPE_CRASH ? "What were you doing when the application crashed?\n" : "",
 				      details_text != NULL ? details_text : "",
 				      gdb_text != NULL ? gdb_text : "<empty backtrace>");
 	
 	email = gtk_entry_get_text (GTK_ENTRY (glade_xml_get_widget (xml, "email-entry")));
-	type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(xml), "type"));
 	title = create_report_title (app, type, details_text);
 
-	message = bugzilla_create_report (app, gnome_version, email, title, final_text, xml, &err);
+	message = bugzilla_create_report (app, type, gnome_version, email, title, final_text, xml, &err);
 	if (message == NULL) {
 		char *text;
 
@@ -668,7 +758,7 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GladeXML
 		g_free (details_text);
 		g_free (title);
 		g_free (final_text);
-		gtk_main_quit();
+		bug_buddy_quit (xml);
 		return;
 	}
 
@@ -718,74 +808,119 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GladeXML
 	g_free (title);
 	g_free (final_text);
 
-	gtk_statusbar_push (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")),
-			    SENDING, _("Sending..."));
+	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (glade_xml_get_widget (xml, "progressbar")),
+				   _("Sending..."));
 
 }
 
+/* A local part is valid if it is one or more valid characters. */
+static gboolean
+email_local_part_is_valid (const char *local_part)
+{
+	const char *character;
 
+	if (!local_part[0])
+		return FALSE;
+
+	for (character = local_part; *character; character++) {
+		/* If character is alphanumeric it is valid. */
+		if (g_ascii_isalnum (*character))
+			continue;
+
+		/* If character is "-", "_" or "." it is valid. */
+		if (*character == '-' || *character == '_' || *character == '.')
+			continue;
+
+		/* Not valid character, not valid local part. */
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+/* A domain label is valid if it is one or more valid characters. */
+static gboolean
+email_domain_label_is_valid (const char *domain_label)
+{
+	const char *character;
+	int i;
+
+	/* Validate each character, whilst measuring length, i. */
+	for (i = 0; *(character = domain_label + i); i++) {
+
+		/* If character is alphanumeric it is valid. */
+		if (g_ascii_isalnum (*character))
+			continue;
+
+		/* If it's a hyphen, it's also valid. */
+		if (*character == '-')
+			continue;
+
+		/* Anything else is invalid */
+		return FALSE;
+	}
+
+	/* Labels must be between 1 and 63 characters long */
+	if (i < 1 || i > 63) {
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+/* A domain is valid if it is one or more valid, dot-separated labels. */
+static gboolean
+email_domain_is_valid (const char *domain)
+{
+	char **labels;
+	char **this_label;
+	
+	/* If there is no domain, there are no domain labels and the domain is
+	 * not valid. */ 
+	if (!domain[0])
+		return FALSE;
+
+	/* Split the domain on the dot to validate labels. */
+	labels = g_strsplit (domain, ".", 0);
+
+	for (this_label = labels; *this_label; this_label++) {
+		if (!email_domain_label_is_valid (*this_label)) {
+			g_strfreev (labels);
+			return FALSE;
+		}
+	}
+
+	g_strfreev (labels);
+	return TRUE;
+}
 
 /* Check for *simple* email addresses of the form user@host, with checks
  * in characters used, and sanity checks on the form of host.
  */
 /* FIXME: Should we provide a useful error message? */
 static gboolean
-email_is_valid (const char *addy)
+email_is_valid (const char *address)
 {
-	const char *tlds[] = {
-		"com",	"org",	"net",	"edu",	"mil",	"gov",	"int",	"arpa",
-		"aero",	"biz",	"coop",	"info",	"museum",	"name", "pro",
-		NULL
-	};
+	char *local_part;
+	char *domain;
+	char **parts;
+	gboolean is_valid;
 
-	gboolean is_dot_ok = FALSE;
-	gboolean seen_at  = FALSE;
-	const char *tld = NULL;
-	const char *s;
-	const char **t;
+	parts = g_strsplit (address, "@", 2);
 
-	for (s = addy; *s; s++) {
-		/* Must be in acceptable ASCII range. */
-		if ((unsigned char)*s <= 32 || (unsigned char)*s >= 128)
-			return FALSE;
-		if (*s == '.') {
-			/* No dots at start, after at, or two in a row. */
-			if (!is_dot_ok)
-				return FALSE;
-			is_dot_ok = FALSE;
-			if (seen_at)
-				tld = s+1;
-		} else if (*s == '@') {
-			/* Only one at, not allowed if a dot isn't. */
-			if (!is_dot_ok || seen_at)
-				return FALSE;
-			seen_at = TRUE;
-			is_dot_ok = FALSE;
-		} else {
-			/* Can put a dot or at after this character. */
-			is_dot_ok = TRUE;
-		}
-	}
-	if (!seen_at || tld == NULL)
+	/* Check we have the 2 parts */
+	if (!(local_part = parts[0]) || !(domain = parts[1])) {
+		g_strfreev (parts);
 		return FALSE;
-
-	/* Make sure they're not root. */
-	if (!strncmp (addy, "root@", 5))
-		return FALSE;
-
-	/* We'll believe any old ccTLD. */
-	if (strlen (tld) == 2)
-		return TRUE;
-	/* Otherwise, check the TLD list. */
-	for (t=tlds; *t; t++) {
-		if (!g_ascii_strcasecmp (tld, *t))
-			return TRUE;
 	}
-	/* Unrecognised. :( */
-	return FALSE;
+
+	is_valid = email_local_part_is_valid (local_part)
+		&& email_domain_is_valid (domain);
+
+	g_strfreev (parts);
+	return is_valid;
 }
 
-
 static void
 check_email (GtkEditable *editable, gpointer data)
 {
@@ -796,7 +931,8 @@ check_email (GtkEditable *editable, gpointer data)
 	gtk_widget_set_sensitive (glade_xml_get_widget (xml, "send-button"),
 				  email_is_valid (email));
 }
-		
+
+	
 static void
 on_send_clicked (GtkWidget *button, gpointer data)
 {
@@ -814,6 +950,7 @@ on_send_clicked (GtkWidget *button, gpointer data)
 
 	gtk_widget_show (glade_xml_get_widget (xml, "progressbar"));
 	gtk_widget_hide (glade_xml_get_widget (xml, "final-box"));
+	gtk_widget_hide (glade_xml_get_widget (xml, "review-box"));
 
 	gtk_widget_size_request (glade_xml_get_widget (xml, "main-window"), &requisition);
 	gtk_window_resize (GTK_WINDOW (glade_xml_get_widget (xml, "main-window")),
@@ -822,13 +959,26 @@ on_send_clicked (GtkWidget *button, gpointer data)
 	send_report (app, gnome_version, xml);
 }
 
-static void
+static gboolean
 gdb_insert_text (const gchar *stacktrace, GladeXML *xml)
 {
 	GtkTextView *text_view;
 	GtkTextIter end;
 	GtkTextBuffer *buffer;
 
+	/* FIXME: These strings are gdb specific, we should add here also dbx */
+	const char *bt_step1 = "#1";
+	const char *bt_step2 = "#2";
+	const char *bt_step3 = "#3";
+
+	if (!g_strrstr (stacktrace, bt_step1) &&
+            !g_strrstr (stacktrace, bt_step2) &&
+            !g_strrstr (stacktrace, bt_step3)) {
+		return FALSE;
+	}
+
+
+
 	text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
 	buffer = gtk_text_view_get_buffer (text_view);
 	gtk_text_buffer_get_end_iter (buffer, &end);
@@ -836,6 +986,7 @@ gdb_insert_text (const gchar *stacktrace, GladeXML *xml)
 	/* add the stacktrace to the GtkTextView */
 	gtk_text_buffer_insert (buffer, &end, stacktrace, strlen (stacktrace));
 
+	return TRUE;
 }
 
 static void
@@ -853,36 +1004,98 @@ show_pending_checkbox_if_pending (GladeXML *xml)
 	g_free (dirname);
 }
 
+static GtkWidget*
+create_debuginfo_link (void)
+{
+	GtkWidget *urlbutton;
+
+	/* create a clickable link to the bug report */
+	urlbutton = gtk_link_button_new_with_label ("http://live.gnome.org/GettingTraces/DistroSpecificInstructions", 
+						    _("Getting useful crash reports"));
+	g_signal_connect (G_OBJECT (urlbutton), "clicked", G_CALLBACK (link_callback), NULL);
+
+	return urlbutton;
+}
+
+
+static void
+useless_finished (GladeXML *xml)
+{
+
+	GtkWidget *button, *image, *main_vbox, *urlbutton;
+	BugzillaApplication *app;
+	char *label_text;
+
+	app = g_object_get_data (G_OBJECT (xml), "app");
+
+	gtk_widget_hide (glade_xml_get_widget (xml, "progressbar"));
+
+	label_text = g_markup_printf_escaped (_("The application %s crashed. The bug reporting tool was "
+						"unable to collect enough information about the crash to be "
+						"useful to the developers.\n\n"
+						"In order to submit useful reports, please consider installing "
+						"debug packages for your distribution.\n"
+						"Click the link below to get information about how to install "
+						"these packages:\n"),
+			      			app->name);
+	gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")),
+		    	label_text);
+
+	main_vbox = glade_xml_get_widget (xml, "main-vbox");
+	urlbutton = create_debuginfo_link ();
+	gtk_box_pack_end (GTK_BOX (main_vbox), urlbutton, FALSE, FALSE, 0);
+
+	gtk_widget_show (urlbutton);
+
+	g_free (label_text);
+
+	button = glade_xml_get_widget (xml, "close-button");
+	gtk_button_set_label (GTK_BUTTON (button), _("_Close"));
+	gtk_button_set_use_underline (GTK_BUTTON (button), TRUE);
+		
+	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON), 
+	gtk_button_set_image (GTK_BUTTON (button), image);
+
+	gtk_widget_hide (glade_xml_get_widget (xml, "email-entry"));
+}
+
+
 static void
 known_app_finished (GladeXML *xml)
 {
+	BugzillaApplication *app;
 	GtkWidget *email_entry;
 	GtkWidget *button;
 	char *default_email;
-	char *lang_note, *label_text;
-	const char *en_lang_note = N_("\nPlease write your report in English, if possible.\n");
+	char *lang_note, *label_text, *s;
+	const char *en_lang_note = N_("\n\nPlease write your report in English, if possible.");
 
+	app = g_object_get_data (G_OBJECT (xml), "app");
+
+	fill_custom_info (app, xml);
 	fill_stderr_info (xml);
 	
 	gtk_widget_show (glade_xml_get_widget (xml, "final-box"));
 	gtk_widget_show (glade_xml_get_widget (xml, "send-button"));
 	gtk_widget_hide (glade_xml_get_widget (xml, "progressbar"));
-	gtk_statusbar_pop (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")), DEBUGGING);
 
 	lang_note = gettext (en_lang_note);
 
-	label_text = g_strconcat (_("The information about the crash has been successfully collected. "
-				    "Please provide some background about what you were doing when "
+	label_text = g_strconcat (_("Information about the %s application crash has been successfully collected.  "
+				    "Please provide some more details about what you were doing when "
 				    "the application crashed.\n\n"
 
-				    "A valid email address is required.  This will allow developers to "
-				    "contact you for more information if necessary.\n\n"),
+				    "A valid email address is required.  This will allow the developers to "
+				    "contact you for more information if necessary."),
 				  strcmp (lang_note, en_lang_note) ? lang_note : NULL,
 				  NULL);
 
+	s = g_markup_printf_escaped (label_text, app->name);
+	
 	gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")),
-			    label_text);
+			    s);
 
+	g_free (s);
 	g_free (label_text);
 
 	show_pending_checkbox_if_pending (xml);
@@ -903,6 +1116,14 @@ known_app_finished (GladeXML *xml)
 		gtk_widget_set_sensitive (button, FALSE);
 	}
 
+	if (search_forbidden_words (xml)) {
+		char *review_text = g_markup_printf_escaped ("<small><i><span weight=\"bold\">%s</span>  %s</i></small>",
+							     _("WARNING:"),		
+							     _("Some sensitive data is likely present in the crash details.  "
+							       "Please review and edit the information if you are concerned "
+							       "about transmitting passwords or other sensitive data."));
+		gtk_label_set_markup (GTK_LABEL (glade_xml_get_widget (xml, "review-label")), review_text);
+	}
 	g_signal_connect (glade_xml_get_widget (xml, "review-button"), "clicked", G_CALLBACK (show_review), xml);
 	gtk_widget_show (glade_xml_get_widget (xml, "review-box"));
 
@@ -914,8 +1135,11 @@ gdb_finished (const gchar *stacktrace, gpointer data)
 {
 	GladeXML *xml = (GladeXML*) data;
 
-	gdb_insert_text (stacktrace, xml);
-	known_app_finished (xml);
+	if (gdb_insert_text (stacktrace, xml)) {
+		known_app_finished (xml);
+	} else {
+		useless_finished (xml);
+	}
 }
 
 
@@ -969,7 +1193,7 @@ on_save_clicked (GtkWidget *button, gpointer user_data)
 
 	gtk_widget_destroy (dialog);
 	if (saved) {
-		gtk_main_quit ();
+		bug_buddy_quit (xml);
 	}
 }
 
@@ -992,16 +1216,15 @@ unknown_app_finished (GladeXML *xml)
 	
 	/* make the send button into a save button :-) */
 	button = glade_xml_get_widget (xml, "send-button");
-	gtk_button_set_label (GTK_BUTTON (button), _("Save"));
+	gtk_button_set_label (GTK_BUTTON (button), _("_Save Bug Report"));
+	gtk_button_set_use_underline (GTK_BUTTON (button), TRUE);
 	g_signal_connect (GTK_BUTTON (button), "clicked", G_CALLBACK (on_save_clicked), xml);
 	gtk_widget_show (glade_xml_get_widget (xml, "send-button"));
 
-	gtk_statusbar_pop (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")), DEBUGGING);
-
 	gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")),
-			    _("The information about the crash has been successfully collected.\n\n"
+			    _("Information about the crash has been successfully collected.\n\n"
 	                      "The application that crashed is not known to bug-buddy, therefore the "
-	                      "bug report can not be sent to the GNOME Bugzilla.  Please save the "
+	                      "bug report cannot be sent to the GNOME Bugzilla.  Please save the "
 	                      "bug to a text file and report it to the appropriate bug tracker "
 	                      "for this application."));
 
@@ -1021,13 +1244,9 @@ gdb_finished_unknown_app (const gchar *stacktrace, gpointer data)
 	unknown_app_finished (xml);
 }
 
-
-
-
 static void
-close_callback (GtkWidget *widget, gpointer user_data)
+bug_buddy_quit (GladeXML *xml)
 {
-	GladeXML *xml = (GladeXML *)user_data;
 	gpointer data;
 
 	g_return_if_fail (xml != NULL);
@@ -1045,9 +1264,21 @@ close_callback (GtkWidget *widget, gpointer user_data)
 		}
 	}
 
+	g_hash_table_destroy (apps);
+
+	g_object_unref (xml);
+
 	gtk_main_quit ();
 }
 
+static void
+close_callback (GtkWidget *widget, gpointer user_data)
+{
+	GladeXML *xml = (GladeXML *)user_data;
+
+	bug_buddy_quit (xml);
+}
+
 static void
 help_callback (GtkWidget *widget, gpointer user_data)
 {
@@ -1385,7 +1616,6 @@ main (int argc, char *argv[])
 	GtkIconInfo *icon_info;
 	GladeXML *xml;
 	char *s;
-	GHashTable *apps;
 	char *bt;
 	GtkTextView *text_view;
 	BugzillaApplication *app;
@@ -1437,15 +1667,13 @@ main (int argc, char *argv[])
 
 	main_window = glade_xml_get_widget (xml, "main-window");
 	g_signal_connect (main_window, "delete-event", G_CALLBACK (delete_callback), xml);
-	gtk_window_set_default_size (GTK_WINDOW (main_window), 400, 400);
 
 	gtk_widget_hide (glade_xml_get_widget (xml, "final-box"));
 
-	statusbar = glade_xml_get_widget (xml, "statusbar");
-	gtk_statusbar_push (GTK_STATUSBAR (statusbar),
-			    LOADING_INFO, _("Collecting info from your system..."));
 	progress = g_timeout_add (100, update_progress_bar,
 				  glade_xml_get_widget (xml, "progressbar"));
+	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (glade_xml_get_widget (xml, "progressbar")),
+				   _("Collecting information from your system..."));
 
 	if (gopt_data.app_file == NULL && gopt_data.package == NULL) {
 		buddy_error (NULL, _("Either --appname or --package arguments are required.\n"));	
@@ -1485,8 +1713,8 @@ main (int argc, char *argv[])
 
 		/* we handle an unknown application (no .desktop file) differently */
 		if (app != NULL) {
-			s = g_markup_printf_escaped (_("The application %s has crashed.\n"
-		                               		"We are collecting information about the crash to send it to the\n"
+			s = g_markup_printf_escaped (_("The %s application has crashed.  "
+		                               		"We are collecting information about the crash to send to the "
                                                		"developers in order to fix the problem."), app->name);	
 			gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")), s);
 			g_free (s);
@@ -1503,11 +1731,10 @@ main (int argc, char *argv[])
 				}
 			}
 			fill_gnome_info (app, gnome_version, xml);
-			fill_custom_info (app, xml);
 		}
 
-		gtk_statusbar_push (GTK_STATUSBAR (statusbar),
-			    	    DEBUGGING, _("Collecting info from the crash..."));
+		gtk_progress_bar_set_text (GTK_PROGRESS_BAR (glade_xml_get_widget (xml, "progressbar")),
+					   _("Collecting information from the crash..."));
 
 		fill_system_info (xml);
 		
@@ -1547,7 +1774,7 @@ main (int argc, char *argv[])
 		GtkWidget *email_entry;
 		char *default_email;
  		char *msg, *lang_note;
-                const char *en_lang_note = N_("\nPlease write your report in English, if possible.\n");
+                const char *en_lang_note = N_("\n\nPlease write your report in English, if possible.");
 		
 		app = g_hash_table_find (apps, (GHRFunc)bugzilla_search_for_package, gopt_data.package);
 		if (app == NULL) {
@@ -1562,7 +1789,7 @@ main (int argc, char *argv[])
 
 		lang_note = gettext (en_lang_note);
 		msg = g_strconcat (_("Thank you for helping us improving our software.\n"
-				     "Please fill your suggestions/error information for %s application.\n"),
+				     "Please fill your suggestions/error information for %s application."),
 				   strcmp (lang_note, en_lang_note)? lang_note: NULL, 
 				   NULL);
 
@@ -1591,12 +1818,11 @@ main (int argc, char *argv[])
 		gtk_widget_show (glade_xml_get_widget (xml, "final-box"));
 		gtk_widget_show (glade_xml_get_widget (xml, "send-button"));
 		gtk_widget_hide (glade_xml_get_widget (xml, "progressbar"));
-		gtk_statusbar_pop (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")), DEBUGGING);
 
 		s = g_markup_printf_escaped (_("Thank you for helping us improving our software.\n"
 					        "Please fill your suggestions/error information for %s application.\n\n"
 			      			"A valid email address is required.  This will allow developers to "
-			      			"contact you for more information if necessary.\n\n"),
+			      			"contact you for more information if necessary."),
 				             app->name);
 		gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")), s);
 		g_free (s);
@@ -1632,8 +1858,7 @@ main (int argc, char *argv[])
 	
 	gtk_main ();
 
-	g_hash_table_destroy (apps);
-
 	g_object_unref (program);
+
 	return 0;
 }
